// Copyright 2011 Zero and One Computing, Inc
// Contact yggy@zeroandone.ca
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// UnitEd.cs
//
// Launch program for arbitrary external editors in Unity.
//
// TODO: notes on how to compile, install, and use.
// To make this work nicely ...
// 1. need editor launcher, built with a MonoDevelop project, and pre-compiled for ease of use
// 2. in-Unity installer copies the launcher over top of MonoDevelop.exe (preserving a backup!)(browse to find MonoDevelop if can't auto-detect)
// 3. custom editor window to browse to editor and configure command line (sets UNITYEDITOR environment variable)
// 4. remembers your selections and lets you toggle between them
// 5. pre-configured with command lines for popular editors (MonoDevelop, Notepad++, VCSExpress, UniScite, ...)

// what about Mac support? will need community input
//
// History:
// Yggy King / October 1, 2010 - first version
// Yggy King / December 14, 2011 - first sent into the wild, to Edy Tado <edytado@gmail.com>

using System;
using System.Windows.Forms;
using System.Diagnostics;
using System.Reflection;

[assembly:AssemblyTitle("UnitEd (set UNITYEDITOR to control)")]

class Program
{
	static void Main(string[] args)
	{
		string file = String.Empty;
		string line = "1";
		bool debug = false;
		
		
		string fullcmdline = "";
		
		foreach (string arg in args)
		{
			fullcmdline += arg + " \n";
		
            // Handle UniSciTE's -goto parameter.
			if (arg.StartsWith("-goto"))
			{
				line = arg.Substring(arg.IndexOf(":") + 1);
			}
			else if (arg == "-debug")
			{
				debug = true;
			}
			else
			{
				file = arg.Trim('"');
                
                // Handle MonoDevelop's trailing ";<line>" syntax.
                int iSemicolon = file.LastIndexOf(';');
                if (iSemicolon > 0)
                {
                    line = file.Substring(iSemicolon + 1);
                    file = file.Substring(0, iSemicolon);
                }
                
				// Wrap filename in quotes.
				file = String.Format("\"{0}\"", file);
			}
		}

		if (file != String.Empty)
		{
			string UNITYEDITOR = Environment.GetEnvironmentVariable("UNITYEDITOR");
			if (String.IsNullOrEmpty(UNITYEDITOR))
			{
				MessageBox.Show("UNITYEDITOR environment variable not defined.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			string editor = UNITYEDITOR;
			if (UNITYEDITOR.Contains("-debug"))
			{
				debug = true;
				editor = editor.Replace("-debug", "");
			}
			string cmdline = "";
			if (editor.StartsWith("\""))
			{
				int iQuote = editor.IndexOf('"', 1);
				if (iQuote > 0)
				{
					cmdline = editor.Substring(iQuote + 1).Trim();
					editor = editor.Substring(1, iQuote - 1).Trim();
				}
			}
			else
			{
				int iQuote = editor.IndexOf(" ");
				if (iQuote > 0)
				{
					cmdline = editor.Substring(iQuote + 1).Trim();
					editor = editor.Substring(0, iQuote).Trim();
				}
			}			
			
			if (String.IsNullOrEmpty(editor) && !debug)
			{
				MessageBox.Show(string.Format("UNITYEDITOR environment variable does not specify an editor.\nUNITYEDITOR={0}", UNITYEDITOR), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			if (String.IsNullOrEmpty(cmdline))
			{
				cmdline = file;
			}
			else
			{
				cmdline = cmdline.Replace("{line}", line);				
				if (cmdline.Contains("{file}"))
				{
					cmdline = cmdline.Replace("{file}", file);
				}
				else
				{
					cmdline = String.Format("{0} {1}", cmdline, file);
				}
			}

            string unityArgs = string.Join(" ", args);
            string msg = String.Format("Unity args: {0}\nUNITYEDITOR={1}\nEditor: {2}\nArguments: {3}", unityArgs, UNITYEDITOR, editor, cmdline);
			if (debug)
			{
				if (MessageBox.Show(msg, "Debug", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
				{
					return;
				}
			}
            try
            {
                Process.Start(editor, cmdline);
            }
            catch (Exception ex)
            {
				string msg2 = String.Format("{0}\n\n{1}", msg, ex.ToString());
				MessageBox.Show(msg2, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
		}
	}
}
